<?php

/**
 * Admin controller.
 */
class ControllerExtensionPaymentConotoxiaPay extends Controller
{
    /**
     * Name of plugin.
     */
    const PLUGIN_NAME = 'conotoxiapay';

    /**
     * Conotoxia payment url.
     */
    const CONOTOXIAPAY_HOST = 'https://login.cinkciarz.pl/';

    /**
     * Conotoxia OIDC url.
     */
    const CONOTOXIA_OIDC = 'https://partner.cinkciarz.pl/';

    /**
     * Test payment url for developers.
     */
    const PAYMENTS_HOST_SANDBOX = 'https://pay-api.dev.cin.pl/';

    /**
     * Test OIDC url for developers.
     */
    const OIDC_SANDBOX = 'https://oidc.dev.cin.pl/';

    /**
     * @var array List of errors
     */
    private $error = [];

    /**
     * @var array List of settings
     */
    private $settings = [];

    /**
     * Constructor of admin controller.
     *
     * @param Registry $registry Registry object of OpenCart
     */
    public function __construct($registry)
    {
        parent::__construct($registry);
        $this->settings = [
            $this->getConfigKey('client_id') => '',
            $this->getConfigKey('client_secret') => '',
            $this->getConfigKey('point_of_sale') => '',
            $this->getConfigKey('rsa_pub_key') => '',
            $this->getConfigKey('rsa_priv_key') => '',
            $this->getConfigKey('sandbox') => '',
            $this->getConfigKey('sort_order') => 1,
            $this->getConfigKey('status') => 0,
            $this->getConfigKey('payment_host') => '',
            $this->getConfigKey('payment_oidc') => '',
            $this->getConfigKey('status_completed') => 5,
            $this->getConfigKey('status_rejected') => 7,
            $this->getConfigKey('status_processing') => 2,
            $this->getConfigKey('status_pending') => 4,
            $this->getConfigKey('status_return') => 3,
        ];

        $this->logger = new Log('conotoxiapay.log');
    }

    /**
     * Returns full path of Conotoxia payment extension.
     *
     * @return string
     */
    private function getExtensionName(): string
    {
        return 'extension/payment/' . self::PLUGIN_NAME;
    }

    /**
     * Get full name of value used in configuration
     * @param string $key
     * @return string
     */
    private function getConfigKey(string $key): string
    {
        return 'payment_' . self::PLUGIN_NAME . '_' . $key;
    }

    /**
     * Default action, called on settings page.
     */
    public function index()
    {
        $this->load->language('extension/payment/' . self::PLUGIN_NAME);
        $this->load->model('setting/setting');

        $this->document->setTitle($this->language->get('heading_title'));

        if ($this->request->server['REQUEST_METHOD'] == 'POST' && $this->validate()) {
            $this->load->model('setting/setting');
            foreach ($this->settings as $key => $value) {
                if (isset($this->request->post[$key])) {
                    $this->settings[$key] = $this->request->post[$key];
                }
            }

            if(isset($_POST[$this->getConfigKey('sandbox')])) {
                $this->settings[$this->getConfigKey('payment_host')] = static::PAYMENTS_HOST_SANDBOX;
                $this->settings[$this->getConfigKey('payment_oidc')] = static::OIDC_SANDBOX;
            }else {
                $this->settings[$this->getConfigKey('payment_host')] = static::CONOTOXIAPAY_HOST;
                $this->settings[$this->getConfigKey('payment_oidc')] = static::CONOTOXIA_OIDC;
            }

            $this->model_setting_setting->editSetting('payment_' . self::PLUGIN_NAME, $this->settings);
            $this->session->data['success'] = $this->language->get('text_success');

            $conotoxiaPayment = new ConotoxiaPayment($this->registry);
            if(isset($this->request->post[$this->getConfigKey('rsa_priv_key')]) && isset($this->request->post[$this->getConfigKey('rsa_pub_key')])){
                try {
                    $conotoxiaPayment->savePublicKeyToStorage();
                    $conotoxiaPayment->setPublicKey();
                } catch (Exception $e) {
                    $this->logger->write($this->language->get('key_error_message'));
                    $this->session->data['error'] = $this->language->get('key_error_message');
                }
            }


            $this->response->redirect($this->url->link($this->getExtensionName(),
                'user_token=' . $this->session->data['user_token'], 'SSL'));
        }

        if (isset($this->session->data['success'])) {
            $data['success_msg'] = $this->session->data['success'];
            unset($this->session->data['success']);
        } else {
            $data['success_msg'] = '';
        }

        $data['plugin_name'] = self::PLUGIN_NAME;
        $data['plugin_field_name'] = 'payment_' . self::PLUGIN_NAME;

        $data['heading_title'] = $this->language->get('heading_title');
        $data['breadcrumbs'] = $this->getBreadcrumbs();
        $data['main_header_edit'] = $this->language->get('main_header_edit');
        $data['tab_main'] = $this->language->get('tab_main');
        $data['tab_channels'] = $this->language->get('tab_channels');
        $data['text_active_status'] = $this->language->get('text_active_status');
        $data['conotoxiapay_status'] = (isset($this->request->post[$this->getConfigKey('status')]) ?
            $this->request->post[$this->getConfigKey('status')] : $this->config->get($this->getConfigKey('status')));
        $data['text_sort_order'] = $this->language->get('text_sort_order');
        $data['conotoxiapay_sort_order'] = (isset($this->request->post[$this->getConfigKey('sort_order')]) ?
            $this->request->post[$this->getConfigKey('sort_order')] : $this->config->get($this->getConfigKey('sort_order')));
        $data['conotoxiapay_sandbox'] = (isset($this->request->post[$this->getConfigKey('sandbox')]) ?
            $this->request->post[$this->getConfigKey('sandbox')] : $this->config->get($this->getConfigKey('sandbox')));
        $data['conotoxiapay_rsa_priv_key'] = (isset($this->request->post[$this->getConfigKey('rsa_priv_key')]) ?
            $this->request->post[$this->getConfigKey('rsa_priv_key')] : $this->config->get($this->getConfigKey('rsa_priv_key')));
        $data['conotoxiapay_rsa_pub_key'] = (isset($this->request->post[$this->getConfigKey('rsa_pub_key')]) ?
            $this->request->post[$this->getConfigKey('rsa_pub_key')] : $this->config->get($this->getConfigKey('rsa_pub_key')));
        $data['conotoxiapay_point_of_sale'] = (isset($this->request->post[$this->getConfigKey('point_of_sale')]) ?
            $this->request->post[$this->getConfigKey('point_of_sale')] : $this->config->get($this->getConfigKey('point_of_sale')));
        $data['conotoxiapay_client_secret'] = (isset($this->request->post[$this->getConfigKey('client_secret')]) ?
            $this->request->post[$this->getConfigKey('client_secret')] : $this->config->get($this->getConfigKey('client_secret')));
        $data['conotoxiapay_client_id'] = (isset($this->request->post[$this->getConfigKey('client_id')]) ?
            $this->request->post[$this->getConfigKey('client_id')] : $this->config->get($this->getConfigKey('client_id')));
        $data['text_enabled'] = $this->language->get('text_enabled');
        $data['text_disabled'] = $this->language->get('text_disabled');
        $data['cancel'] = $this->createUrl('extension/marketplace&type=payment');
        $data['button_save'] = $this->language->get('button_save');
        $data['button_cancel'] = $this->language->get('button_cancel');

        $data['error'] = (!empty($this->error) ? $this->error : null);

        $data['header'] = $this->load->controller('common/header');
        $data['column_left'] = $this->load->controller('common/column_left');
        $data['footer'] = $this->load->controller('common/footer');

        $this->response->setOutput($this->load->view($this->getExtensionName(), $data));
    }

    /**
     * Action called during install this plugin.
     */
    public function install()
    {

        $this->load->model('setting/setting');
        $this->model_setting_setting->editSetting('payment_' . self::PLUGIN_NAME, $this->settings);
    }

    /**
     * Action called during uninstall this plugin.
     */
    public function uninstall()
    {

        $this->load->model('setting/setting');
        $this->model_setting_setting->deleteSetting('payment_' . self::PLUGIN_NAME);
    }

    /**
     * Checks, if settings data is validate, before a saving of changes.
     *
     * @return bool
     */
    public function validate()
    {
        if (!$this->user->hasPermission('modify', $this->getExtensionName())) {
            $this->error['permission'] = $this->language->get('error_permission');
        }
        if (!$this->request->post[$this->getConfigKey('client_id')]) {
            $this->error['client_id'] = $this->language->get('error_client_id');
        }
        if (!$this->request->post[$this->getConfigKey('client_secret')]) {
            $this->error['client_secret'] = $this->language->get('error_client_secret');
        }
        if (!$this->request->post[$this->getConfigKey('point_of_sale')]) {
            $this->error['point_of_sale'] = $this->language->get('error_point_of_sale');
        }
        if (!$this->request->post[$this->getConfigKey('rsa_pub_key')]) {
            $this->error['rsa_pub_key'] = $this->language->get('error_rsa_pub_key');
        }
        if (!$this->request->post[$this->getConfigKey('rsa_priv_key')]) {
            $this->error['rsa_priv_key'] = $this->language->get('error_rsa_priv_key');
        }

        return !$this->error;
    }

    /**
     * Returns array of breadcrumbs for settings page.
     *
     * @return array
     */
    private function getBreadcrumbs()
    {
        $breadcrumbs = [];

        $breadcrumbs[] = [
            'href' => $this->createUrl('common/dashboard'),
            'text' => $this->language->get('text_home'),
            'separator' => false,
        ];

        $breadcrumbs[] = [
            'href' => $this->createUrl('marketplace/extension&type=payment'),
            'text' => $this->language->get('text_payment'),
            'separator' => ' :: ',
        ];

        $breadcrumbs[] = [
            'href' => $this->createUrl($this->getExtensionName()),
            'text' => $this->language->get('heading_title'),
            'separator' => ' :: ',
        ];

        return $breadcrumbs;
    }

    /**
     * Create an url with given value as 'route' parameter
     * @param string $route
     * @return string
     */
    private function createUrl(string $route): string
    {
        return HTTPS_SERVER . 'index.php?route=' . $route . '&user_token=' . $this->session->data['user_token'];
    }
}
