<?php

use CKPL\Pay\Pay;

/**
 * Class PaymentFactory is responsible for building the payment
 */
class PaymentFactory
{
    /**
     * @param Pay $pay
     * @param array $order
     * @return \CKPL\Pay\Definition\Payment\PaymentInterface
     * @throws \CKPL\Pay\Exception\Definition\AmountException
     * @throws \CKPL\Pay\Exception\Definition\PaymentCustomerException
     * @throws \CKPL\Pay\Exception\Definition\PaymentException
     */
    public static function create(Pay $pay, array $order)
    {
        $paymentBuilder = $pay->payments()->createPaymentBuilder();
        $orderBuilder = $paymentBuilder->createOrderBuilder();
        $customerBuilder = $orderBuilder->createCustomerBuilder();

        $amount = $paymentBuilder->createAmountBuilder()
            ->setValue($order['total'])
            ->setCurrency($order['currency_code'])
            ->getAmount();

        $paymentOrder = $orderBuilder
            ->setCustomer(
                $customerBuilder
                    ->setFirstName($order['firstname'])
                    ->setLastName($order['lastname'])
                    ->setEmail($order['email'])
                    ->setPhone($order['telephone'])
                    ->getPaymentCustomer()
            )
            ->getPaymentOrder();

        return $paymentBuilder
            ->setExternalPaymentId($order['order_id'])
            ->setAmount($amount)
            ->setDescription($order['store_url'] . ' #' . $order['order_id'])
            ->setOrder($paymentOrder)
            ->getPayment();
    }
}